from collections.abc import Iterable
from os import environ
import subprocess


def sub_run(
    *args,
    capture=False,
    returns: None | Iterable[int] = None,
    **kwargs,
) -> subprocess.CompletedProcess:
    kwargs.setdefault('check', not bool(returns))
    capture = kwargs.setdefault('capture_output', capture)
    args = args + kwargs.pop('args', ())
    env = kwargs.pop('env', None)
    if env:
        kwargs['env'] = environ | env
    if capture:
        kwargs.setdefault('text', True)

    try:
        result = subprocess.run(args, **kwargs)
        if returns and result.returncode not in returns:
            raise subprocess.CalledProcessError(result.returncode, args[0])
        return result
    except subprocess.CalledProcessError as e:
        if capture:
            print(e.stderr)
        raise



"""
This conftest mostly for handling warnings.  Use the other conftest.py files for app/test config.

- Filters for warnings that are triggered during import go at the top level.
- Filters for warnings thrown during test runs goes in pytest_configure() below.

Having two conftest.py files is necessary because the warning configuration needs to happen before
the application's tests and/or code have a chance to import other libraries which may trigger
warnings.  So this file remains a filesystem level above the "real" conftest.py which does all the
imports.
"""

import warnings


# Treat any warning issued in a test as an exception so we are forced to explicitly handle or
# ignore it.
warnings.filterwarnings('error')
# Examples:
# warnings.filterwarnings(
#     'ignore',
#     "'cgi' is deprecated and slated for removal in Python 3.13",
#     category=DeprecationWarning,
#     module='webob.compat',
# )
# warnings.filterwarnings(
#     'ignore',
#     "'crypt' is deprecated and slated for removal in Python 3.13",
#     category=DeprecationWarning,
#     module='passlib.utils',
# )
###########
# REMINDER: when adding an ignore, add an issue to track it
###########


def pytest_configure(config):
    """
    You may be able to do all your ignores above.  If you find some warnings need to be ignored
    in pytest, you can do that with something like:

        config.addinivalue_line(
            'filterwarnings',
            # Note the lines that follow are implicitly concatinated, no "," at the end
            'ignore'
            ':pythonjsonlogger.jsonlogger has been moved to pythonjsonlogger.json'
            ':DeprecationWarning'
            ':wtforms.meta',
        )
    """



import os
from pathlib import Path

from ansible.errors import AnsibleError
from ansible.plugins.vars import BaseVarsPlugin


class VarsModule(BaseVarsPlugin):
    def ans_find_vars_file(self, start_path: str):
        current_path = Path(start_path)
        while current_path.parent != current_path:
            if current_path.joinpath('vars_plugins').exists():
                vars_fpath = current_path / 'ans-env-vars.yaml'
                if vars_fpath.exists():
                    return vars_fpath
            current_path = current_path.parent

    def get_vars(self, loader, path, entities, cache=True):
        env_vars_path = self.ans_find_vars_file(path)

        if not env_vars_path:
            raise AnsibleError(f'Expected "ans-env-vars.yaml" to exist in {path} or parent')

        line_items = loader.load_from_file(env_vars_path.as_posix())
        if not isinstance(line_items, list):
            raise AnsibleError(
                'Expected "ans-env-vars.yaml" to be a list of strings',
            )

        missing = []
        result = {}
        for item in line_items:
            if isinstance(item, dict):
                ans_var_name, env_var_name = next(iter(item.items()))
            else:
                assert isinstance(item, str)
                ans_var_name = item
                env_var_name = item

            env_val = os.environ.get(env_var_name)
            if env_val is None:
                missing.append(env_var_name)
            else:
                result[ans_var_name] = env_val

        if missing:
            raise AnsibleError('Required env variables not set: {}'.format(', '.join(missing)))

        return result
